/* 
 *  playdv.c
 *
 *     Copyright (C) Charles 'Buck' Krasic - April 2000
 *     Copyright (C) Erik Walthinsen - April 2000
 *     Modified by Franck Bonin for Capture May 2000
 *
 *  This file is part of Capture and libdv, a free DV (IEC 61834/SMPTE 314M)
 *  decoder.
 *
 *  libdv is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your
 *  option) any later version.
 *   
 *  libdv is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 *  The libdv homepage is http://libdv.sourceforge.net/.  
 */

#include <glib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>

#include "playdv.h"
#include "audio.h"

void playdv_audio(capture_t* capture) {
  gboolean isPAL = 0;
  gboolean is61834 = 0;
  gint numDIFseq;
  gint ds;
  gint v;
  guint dif;
  guint offset;

  guint deplacement, nb_lu; // pour simuluer les fseek
  static guint8 rien[2000] __attribute__ ((aligned (64)));  // buffer de lecture des donnees qui ne sont pas de la video
  static int aud_block = 0; // pour le son
  static guint8 audbuffer[80]	__attribute__ ((aligned (64))); // buffer audio

  triple_buff_t *f = (capture->b_son);

  /* init sound */
  aud_init_sound();

  dif = 0;
  nb_lu = 0;
  while (!(capture->g_alldone)) {  

    /* Rather read a whole frame at a time, as this gives the
       data-cache a lot longer time to stay warm during decode */

    // start by reading header block, to determine stream parameters
    offset = dif * 80;
    deplacement = (offset - nb_lu);
    if (deplacement) {
      if (bread(f,NULL,deplacement)<1) break;
      nb_lu+=deplacement;
    }

    if (bread(f,rien,400)<1) break; 
    nb_lu+=400;
    isPAL = rien[3] & 0x80;
    is61834 = rien[3] & 0x01;
    numDIFseq = isPAL ? 12 : 10;

    /* init frame sound NTSC ONLY*/
    aud_init_frame(525); //PAL = 625

    // each DV frame consists of a sequence of DIF segments 
    for (ds=0; ds<numDIFseq; ds++) { 
      // Each DIF segment conists of 150 dif blocks, 135 of which are video blocks
      aud_block = 0;
      dif += 6; // skip the first 6 dif blocks in a dif sequence 

      /*
	A video segment consists of 5 video blocks, where each video
	block contains one compressed macroblock.  DV bit allocation
	for the VLC stage can spill bits between blocks in the same
	video segment.  So parsing needs the whole segment to decode
	the VLC data
      */

      // Loop through video segments 
      for (v=0;v<27;v++) {

	// on decode le son - interleaved before every 3rd video segment
	if(!(v % 3)) {
	  offset = dif * 80;
	  deplacement = (offset - nb_lu);
	  if (deplacement) {
	    if(bread(f,NULL,deplacement)<1) break;
	    nb_lu += deplacement;
	  }
	  if(bread(f,audbuffer,80)<1) break; nb_lu += 80;
	  aud_decode_block(audbuffer, ds, aud_block++);
	  dif++;
	} 

	// on passe la video
	offset = dif * 80;
	deplacement = (offset - nb_lu) + 400;
	if(bread(f, NULL, deplacement)<1) break;
	nb_lu += deplacement;
	dif += 5;

     } // for s
    } // ds
    aud_end_frame(); //ecriture du son
  }  
}
