/*
 * triple_buffer.c -- store DV Data and sometime lose DV data
 *
 * Copyright (C) May 2000 Bonin Franck <boninf@free.fr>.
 *
 * This file is part of Capture 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "triple_buffer.h"

int init_3_buffer(triple_buff_t* b, int taille_buffer){
  b->taille_buffer = taille_buffer;
  b->bloquer_lecture = 1;
  pthread_mutex_init(&b->mutex, NULL);
  sem_init(&(b->bloque_lecture), 0, 0);
  b->ecrit = (unsigned char*)malloc(taille_buffer);
  b->offset_ecrit = 0;
  b->lu = (unsigned char*)malloc(taille_buffer);
  b->offset_lu = taille_buffer;
  b->pret = (unsigned char*)malloc(taille_buffer);
  b->init=1;
  return 0;
}

int close_3_buffer(triple_buff_t* b){
  pthread_mutex_lock(&(b->mutex));
  if (b->ecrit) { free(b->ecrit); b->ecrit = 0;}
  if (b->lu) { free(b->lu); b->lu = 0;}
  if (b->pret) { free(b->pret); b->pret = 0;}
  b->init = 0;
  pthread_mutex_unlock(&(b->mutex));  
  return 0;
}

size_t bread(triple_buff_t* b, unsigned char* dest, size_t count) {
  
  if (b->offset_lu == b->taille_buffer) {
    unsigned char *tmp;
    // faut-il attendre ?
    sem_wait(&(b->bloque_lecture));
    pthread_mutex_lock(&(b->mutex));
    // echange buffer
    tmp = b->lu;
    b->lu = b->pret;
    b->pret = tmp;
    // bloque le prochain echange
    b->bloquer_lecture = 1;
    // on recommence au dbut
    b->offset_lu = 0;
    pthread_mutex_unlock(&(b->mutex));  
  }
  if (dest) memcpy(dest, (b->lu + b->offset_lu), count);
  b->offset_lu += count;
  return count;
}

size_t bwrite(triple_buff_t* b, unsigned char* src, size_t count) {
  if (b->offset_ecrit == b->taille_buffer) { /* Vient on de remplire un bloc ?*/ /** Is buffer full ? **/ 
    unsigned char *tmp;
    pthread_mutex_lock(&(b->mutex));
    // echange buffer
    tmp = b->ecrit;
    b->ecrit = b->pret;
    b->pret = tmp;
    b->offset_ecrit = 0;
    if (b->bloquer_lecture) {
      b->bloquer_lecture = 0;
      pthread_mutex_unlock(&(b->mutex));
      sem_post(&(b->bloque_lecture)); /* On libere la lecture */ /** Reader can read **/
    } else
      pthread_mutex_unlock(&(b->mutex));
  }
  /* Ecriture des donnees */
  /** Writing data **/
  if (src) memcpy( (b->ecrit + b->offset_ecrit), src, count);
  b->offset_ecrit += count;
  return count;
}
