/*
 * triple_buffer.h -- store DV Data and sometime lose DV data
 *
 * Copyright (C) May 2000 Bonin Franck <boninf@free.fr>.
 *
 * This file is part of Capture 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef TRPLEBUFFH
#define TRIPLEBUFFH

#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <pthread.h>
#include <semaphore.h>
#ifdef __cplusplus
extern "C" {
#endif

typedef struct triple_buff {
  unsigned long taille_buffer;

  int bloquer_lecture;

  unsigned char *ecrit;
  unsigned long offset_ecrit;
  unsigned char *lu;
  unsigned long offset_lu;
  unsigned char *pret;
 
  sem_t bloque_lecture; // arrete la lecture si besoin
  pthread_mutex_t mutex; // controle l'acces aux flags
  int init;

} triple_buff_t ;

int init_3_buffer(triple_buff_t* b, int taille_buffer);

int close_3_buffer(triple_buff_t* b);

size_t bread(triple_buff_t* b, unsigned char* dest, size_t count) ;

size_t bwrite(triple_buff_t* b, unsigned char* src, size_t count) ;

#ifdef __cplusplus
}
#endif
#endif
